#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ------------------------------------------------------------------
// Genereal Purpose defines
// ------------------------------------------------------------------

#ifdef MVS
#define _MAX_PATH                      13  // style is:  DD:NameOfDD
#define DD_NAME_TOO_LONG               1   // error flag
#define MAX_INI_LINE_LENGTH            80  // max length of MVS sysin
#else
#define MAX_INI_LINE_LENGTH            256 // max for distributed plat
#endif

#ifdef UNIX
#define _MAX_PATH                      PATH_MAX // Unix max path length
#endif

#define NULL_STRING                    ""
#define STR_EQ                         0    // string equal indicator

#define SEQUENCE_NUMBERS_YES           'Y'
#define SEQUENCE_NUMBERS_NO            'N'

// ------------------------------------------------------------------
// Prototype definitions
// ------------------------------------------------------------------

int  GetProfileInt( char *, int,    char *, char );
int  GetProfileString( char *, char *, char *,int,char *,char );
static int  ReadLine( FILE *, char *, char );
static void RemovePrefixBlanks ( char * );
static void RemoveTrailingBlanks ( char * );

#pragma subtitle ("GetProfileInt")
#pragma page ()
// ==================================================================
//
// Function Name
//  GetProfileInt
//
// Last date of modification
//  03 Jan 2000
//
// Description
//  This function will retrieve an integer from an initialization
//  file based on the provided field "entry" name.
//
// Input parameters
//  entry - the name of the entry to find the value of
//  def - the default value in the event of a failed read
//  file_name - the name of the .ini file to read from
//  SeqNumFlag - MVS sequence ## in the file?
//
// Output
//  None.
//
// Return Value
//  the value located at entry
//
// Abend Codes
//  None.
//
// Called Routines
//  None.
//
// ------------------------------------------------------------------
int GetProfileInt( char   *entry,
                   int     def,
                   char   *file_name,
                   char    SeqNumFlag)
{
   // ---------------------------------------------------------------
   // Variable declarations.
   // ---------------------------------------------------------------

   auto FILE           *fp;
   auto int             len;
   auto int             i;
   auto char           *ep;
   auto char            buff[MAX_INI_LINE_LENGTH] = {'\0'};
   auto char            value[12];

   // ---------------------------------------------------------------
   // Code section
   // ---------------------------------------------------------------

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   len = strlen(entry);

   // ---------------------------------------------------------------
   // Open up the ini file.
   // ---------------------------------------------------------------

   if ( !(fp = fopen(file_name,"r")) )
   {
      return( def );
   }

   // ---------------------------------------------------------------
   // Move through file 1 line at a time until a the entry
   // is matched or EOF. '[' is section header - end if found.
   // ---------------------------------------------------------------

   do
   {
       if( !ReadLine( fp, buff, SeqNumFlag ) ||
            buff[0] == '\0' ||
            buff[0] == '[')
       {
           fclose( fp );
           return( def );
       }
   }  while( strncmp( buff, entry, len) );

   ep = strrchr(buff,'=');    // Parse out the equal sign
   ep++;

   if( !strlen(ep) )          // No setting?
   {
       return(def);
   }

   // ---------------------------------------------------------------
   // Remove unnecessary blanks
   // ---------------------------------------------------------------

   RemovePrefixBlanks( ep );
   RemoveTrailingBlanks( ep );

   // ---------------------------------------------------------------
   // Copy only numbers - fail on characters
   // ---------------------------------------------------------------

   for(i = 0; isdigit(ep[i]); i++ )
   {
       value[i] = ep[i];
   }

   value[i] = '\0';
   fclose(fp);                // Clean up and return the value

   return( atoi(value) );
}

#pragma subtitle ("GetProfileString")
#pragma page ()
// ==================================================================
//
// Function Name
//  GetProfileString
//
// Last date of modification
//  03 Jan 2000
//
// Description
//  This function will retrieve a string from an initialization
//  file based on the provided field "entry" name.
//
// Input parameters
//  section - the name of the section to search for
//  entry - the name of the entry to find the value of
//  def - default string in the event of a failed read
//  buffer - a pointer to the buffer to copy into
//  buffer_len - the max number of characters to copy
//  file_name - the name of the .ini file to read from
//  SeqNumFlag - MVS sequence ## in the file?
//
// Output
//  None.
//
// Return Value
//  number of characters copied into the supplied buffer
//
// Abend Codes
//  None.
//
// Called Routines
//  None.
//
// ------------------------------------------------------------------
int GetProfileString( char   *entry,
                      char   *def,
                      char   *buffer,
                      int     buffer_len,
                      char   *file_name,
                      char    SeqNumFlag)
{
   // ---------------------------------------------------------------
   // Variable declarations.
   // ---------------------------------------------------------------

   auto FILE           *fp;
   auto int             len = strlen(entry);
   auto char           *ep;
   auto char            buff[MAX_INI_LINE_LENGTH] = {'\0'};

   // ---------------------------------------------------------------
   // Code section
   // ---------------------------------------------------------------

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   // ---------------------------------------------------------------
   // Open up the ini file.
   // ---------------------------------------------------------------

   if ( !(fp = fopen(file_name,"r")) )
   {
        strncpy(buffer,def,buffer_len);
        return(strlen(buffer));
   }

   // ---------------------------------------------------------------
   // Move through file 1 line at a time until a the entry
   // is matched or EOF. '[' is section header - end if found.
   // ---------------------------------------------------------------

   do
   {
      if( !ReadLine(fp, buff, SeqNumFlag) ||
           buff[0] == '\0' ||
           buff[0] == '[' )
       {
           fclose(fp);
           strncpy(buffer,def,buffer_len);
           return(strlen(buffer));
       }
   }  while( strncmp(buff,entry,len) );

   ep = strrchr(buff,'=');    // Parse out the equal sign
   ep++;

   // ---------------------------------------------------------------
   // Copy up to buffer_len chars to buffer
   // ---------------------------------------------------------------

   strncpy(buffer,ep,(buffer_len - 1) );

   // ---------------------------------------------------------------
   // Remove unnecessary blanks
   // ---------------------------------------------------------------

   RemovePrefixBlanks( buffer );
   RemoveTrailingBlanks( buffer );

   // ---------------------------------------------------------------
   // Clean up and return the amount copied
   // ---------------------------------------------------------------
   fclose(fp);

   return( strlen(buffer) );
}

#pragma subtitle ("ReadLine")
#pragma page ()
// ==================================================================
//
// Function Name
//  ReadLine
//
// Last date of modification
//  03 Jan 2000
//
// Description
//  This function will read a line from the specified file.  If the
//  sequence number flag is on then truncate the line at 72 characters.
//
// Input parameters
//  fp - a pointer to the file to be read from
//  bp - a pointer to the copy buffer
//  SeqNumFlag - MVS sequence ## in the file?
//
// Output
//  None.
//
// Return Value
//  TRUE if successful FALSE otherwise
//
// Abend Codes
//  None.
//
// Called Routines
//  None.
//
// ------------------------------------------------------------------
static int ReadLine( FILE    *fp,
                     char    *bp,
                     char     SeqNumFlag )
{
   // ---------------------------------------------------------------
   // Variable declarations.
   // ---------------------------------------------------------------

    auto char    c = '\0';
    auto int     i = 0;

   // ---------------------------------------------------------------
   // Code section
   // ---------------------------------------------------------------

   // ---------------------------------------------------------------
   // Read one line at a time from the source file
   // ---------------------------------------------------------------
   if (!feof (fp) && fgets (bp, MAX_INI_LINE_LENGTH, fp) != 0)
   {
      if ( strchr (bp, '\n') )
      {
          *strchr (bp, '\n') = '\0';
      }
      else
      {
         // ---------------------------------------------------------
         // Make sure the line is at a length that we can handle.
         // ---------------------------------------------------------

         memset( (bp+MAX_INI_LINE_LENGTH-1), 0 , 1 );
      }

      // ------------------------------------------------------------
      // If the sequence number flag is set then truncate the line
      // at column 72.
      // ------------------------------------------------------------

      if ( (SeqNumFlag == 'y') || (SeqNumFlag == 'Y') )
      {
         bp[71] = '\0';   // sequence numbers begin in column 72
      }

      return(1);   //Yes!! we have a line.
   }

   return(0);   // !! no line.
}

#pragma subtitle ("RemovePrefixBlanks")
#pragma page ()
// ==================================================================
//
// Function Name
//  RemovePrefixBlanks
//
// Last date of modification
//  03 Jan 2000
//
// Description
//  This routine will remove any prefix blanks from a string.
//
// Input parameters
//  string - a pointer to a string where prefix blanks are to be
//           removed - if any.
//
// Output
//  None.
//
// Return Value
//  None.
//
// Abend Codes
//  None.
//
// Called Routines
//  None.
//
// ------------------------------------------------------------------
static void RemovePrefixBlanks( char *String )
{
   // ---------------------------------------------------------------
   // Variable declarations.
   // ---------------------------------------------------------------

   auto   int           Length;           // Length of String

   // ---------------------------------------------------------------
   // Code section
   // ---------------------------------------------------------------

   // ---------------------------------------------------------------
   // Find the 1st nonblank character.
   // ---------------------------------------------------------------

   for ( Length = 0; Length <= (int) strlen( String ); Length++ )
   {
       if ( *(String + Length) != ' ' )   // look for a non-blank
       {
          break;                 // get out - we found a non-blank
       }
   }

   // ---------------------------------------------------------------
   // Remove beginning blanks.
   // ---------------------------------------------------------------
   if (Length != 0)
   {
      memcpy( String, String + Length, (strlen(String) - Length ) );
      memset( String + (strlen(String) - Length ), '\0', Length );
   }

   return;
}

#pragma subtitle ("RemoveTrailingBlanks")
#pragma page ()
// ==================================================================
//
// Function Name
//  RemoveTrailingBlanks
//
// Last date of modification
//  03 Jan 2000
//
// Description
//  This routine will remove trailing blanks from a string.
//
// Input parameters
//  string - a pointer to a string where trailing blanks are to be
//           removed - if any.
//
// Output
//  None.
//
// Return Value
//  None.
//
// Abend Codes
//  None.
//
// Called Routines
//  None.
//
// ------------------------------------------------------------------
static void RemoveTrailingBlanks( char *String )
{
   // ---------------------------------------------------------------
   // Variable declarations.
   // ---------------------------------------------------------------

   auto   int           Length;           // Length of String

   // ---------------------------------------------------------------
   // Code section
   // ---------------------------------------------------------------

   // ---------------------------------------------------------------
   // Find the 1st blank beyond the last valid character.
   // ---------------------------------------------------------------

   for ( Length = (strlen( String ) - 1 ); Length >= 0 ; Length-- )
   {
       if ( *(String + Length) != ' ' )     // look for a non-blank
       {
            break;                 // get out - we found a non-blank
       }
   }

   // ---------------------------------------------------------------
   // Set trailing blanks to nulls.
   // ---------------------------------------------------------------

   memset( String + Length + 1, '\0', (strlen(String) - Length) );

   return;
}
